


# Stage 1 Loader

This is the first-stage bootloader which is loaded and run by the MBR. It is responsible for mounting a filesystem and booting the OS.

# Tools

## Makefile

The stage 1's makefile has some special functions for building a disk image. Since the stage 1 depends on the MBR, we need to make a bootable disk image with both. Use the command `make img` to build the disk image. The resulting image is called `disk.img`, which contains the MBR and the stage 1 but no filesystem. See below for setting up an FS on `disk.img`

### Makefile Options:

#### make img

Builds a bootable disk image including MBR and stage1 loader. MBR image must be present in the `mbr` directory, so you'll need to build that first.

#### make debug

This starts a qemu session and runs your code in gdb. It runs thru the MBR and breaks at address 0x7e00, the first instruction in the stage 1 loader (still in real mode). Your code must be compiled and turned into an image using the complicated command below if you mean to include a partitioned disk image. If you don't care about that and only want the MBR + stage 1 bootloader in your disk image (no filesystem), you can use `make img`.

#### make protdebug

Like `make debug`, starts a qemu session and runs your code in gdb, but breaks at the stage1main function in protected mode.

#### make run

Starts a qemu session and boots your image. No debugging.

#### make disassemble

Disassembles the stage 1 bootloader and prints to stdout.

## partition.sh

Repartitions a disk image generated with `make img`. The MBR used by `make img` has a hardcoded partition table set up in mbr.asm. We need to use `fdisk` to regenerate the partition table according to the disk image's size. Annoyingly this has to be done every time we rebuild the stage 1. We can use `partition.sh` to automate this process. Its job is to (1) delete the partition table in the disk image, (2) write a new table that takes up the whole disk. It takes one argument, the disk image to repartition. You can rebuild everything using the following command:

    make && make img && dd if=fat.img of=disk.img seek=2048 conv=notrunc && ./partition.sh disk.img

Here we rebuild the stage 1 bootloader, make an image consisting of the stage 1 and the MBR, and write a FATFS image to the first partition. We need to make `fat.img` separately:

    dd if=/dev/zero of=fat.img bs=1M count=8
    mkfs.vfat fat.img

